//   RW Design
// ==============
// Technical Evaluation & Creativity Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/atashi/design/design.hpp
// -Declarations of the baseline RW Design sub-library

#pragma once

#include "incls_all.hpp"

namespace rw::atashi::design {

//------------------------------------------------------------------------------
/**
 */
class Design_b : public RW {
 public:
  Design_b();                                    // default ctor
  explicit Design_b(std::string const& handle);  // tag-parm'd ctor
};

//------------------------------------------------------------------------------
/**
 */
class Design : public Design_b {
 public:
  Design();                                    // default ctor
  explicit Design(std::string const& handle);  // tag-parm'd ctor

  inline auto beauty() const -> Beauty& { return beauty_; }
  inline auto form() const -> Form& { return form_; }
  inline auto symmetry() const -> Symmetry& { return symmetry_; }

 private:
  unique_ptr<Beauty_b>   _beauty_{make_unique<Beauty_b>("design_beauty")};
  unique_ptr<Form_b>     _form_{make_unique<Form_b>("design_form")};
  unique_ptr<Symmetry_b> _symmetry_{make_unique<Symmetry_b>("design_symmetry")};

  Beauty&   beauty_{reference_to<Beauty>(_beauty_.get())};
  Form&     form_{reference_to<Form>(_form_.get())};
  Symmetry& symmetry_{reference_to<Symmetry>(_symmetry_.get())};
};

}  // namespace rw::atashi::design

//    --=^=--

// "Just like, make waifu."

// rw_atashi_lib.v211225
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
